#include "cxxopts.hpp"
#include "print.hpp"

template <typename T>
std::vector<tof::FrameType> getTypes(T result) {
  std::vector<tof::FrameType> types;

  if (result.count("raw")) types.push_back(tof::FrameType::RAW);
  if (result.count("rawcm")) types.push_back(tof::FrameType::RAW_COMMON);
  if (result.count("common")) types.push_back(tof::FrameType::COMMON);
  if (result.count("rad")) types.push_back(tof::FrameType::RADIAL);
  if (result.count("intensity")) types.push_back(tof::FrameType::INTENSITY);

  if (result.count("cmpx")) {
    types.push_back(tof::FrameType::REAL);
    types.push_back(tof::FrameType::IMAGINARY);
  }
  if (result.count("ampphi")) {
    types.push_back(tof::FrameType::AMPLITUDE);
    types.push_back(tof::FrameType::PHASE);
  }
  if (result.count("xyz") || result.count("xyzi")) {
    types.push_back(tof::FrameType::X);
    types.push_back(tof::FrameType::Y);
    types.push_back(tof::FrameType::Z);
    if (result.count("xyzi")) types.push_back(tof::FrameType::INTENSITY);
  }

  return types;
}

/** Print all streams camera contains */
void print_stream_list(tof::Camera &cam) {
  auto streams = cam.get_stream_list();

  int n = 0;
  for (const auto &stream : streams) {
    std::cout << "List # " << n++ << std::endl;
    std::cout << "type: " << tof::frame_type_to_string(stream.frame_type())
              << std::endl;
    std::cout << "Frame id: " << stream.frame_id() << std::endl;
    std::cout << "frequency: " << stream.modulation_frequency() << std::endl;
    std::cout << "int_times: " << stream.integration_time() << std::endl;
    std::cout << "process: " << stream.process() << std::endl;
    std::cout << std::endl;
  }
}

int main(int argc, char **argv) {
  cxxopts::Options options("ToF API Capture",
                           "Capture frames to csf file using the Cpp API");

  int nframes = 0;
  std::string serial, csfOut, filterConfig;
  // clang-format off
  options.add_options()
    ("h, help", "Help")
    ("o, output", "CSF output file", cxxopts::value<std::string>(csfOut))
    ("n, nframes", "Number of frames to capture", cxxopts::value<int>(nframes))
    ("c, config", "Filter config file", cxxopts::value<std::string>(filterConfig))
    ("s, serial", "Serial number of camera to connect to", cxxopts::value<std::string>(serial))
    ("list", "List cameras on network")
    ("streams", "Display streams on network")
    ("raw", "Capture raw frames")
    ("rawcm", "Capture raw common frames")
    ("common", "Capture common frames")
    ("cmpx", "Capture real and imaginary frames")
    ("ampphi", "Capture amplitude and phase frames")
    ("rad", "Capture radial distance frames")
    ("xyz", "Capture x, y and z frames")
    ("xyzi", "Capture x, y, z and intensity frames")
    ("intensity", "Capture intensity frames")
  ;
  // clang-format on

  auto result = options.parse(argc, argv);

  if (result.count("h") || result.count("help")) {
    std::cout << options.help() << std::endl;
    return 0;
  }

  if (result.count("list")) {
    print_devices();
    return 0;
  }

  tof::ProcessingConfig config(filterConfig.c_str());
  if (result.count("streams")) {
    tof::KeaCamera cam(config, serial.c_str());
    print_stream_list(cam);
    return 0;
  }

  if (csfOut.empty()) {
    std::cout << "Please specify output file\n";
    return -1;
  }

  if (nframes == 0) {
    std::cout << "Please specify number of frames to capture\n";
    return -1;
  }

  auto types = getTypes(result);

  if (types.size() == 0) {
    std::cout << "Please specify types to capture\n";
    return -1;
  }

  tof::KeaCamera cam(config, serial.c_str());
  tof::CsfWriter writer = tof::create_csf_writer_camera(csfOut.c_str(), cam);

  tof::select_streams(cam, types);

  cam.start();

  int frame_cnt{0};
  while (cam.is_streaming() && frame_cnt < nframes) {
    auto frames = cam.get_frames();

    for (const auto &frame : frames) {
      writer.write_frame(frame);
    }

    frame_cnt++;
  }

  cam.stop();

  std::cout << "Wrote " << frame_cnt << " frames to disk\n";
}
